import { Directive, HostListener, ElementRef, Renderer, Output, EventEmitter, Input } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/merge';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/mergeMap';
import 'rxjs/add/operator/takeUntil';
import 'rxjs/add/operator/takeLast';
import { DraggableHelper } from './draggableHelper.provider';
export var Draggable = (function () {
    function Draggable(element, renderer, draggableHelper) {
        this.element = element;
        this.renderer = renderer;
        this.draggableHelper = draggableHelper;
        this.dragAxis = { x: true, y: true };
        this.snapGrid = {};
        this.ghostDragEnabled = true;
        this.dragStart = new EventEmitter();
        this.dragging = new EventEmitter();
        this.dragEnd = new EventEmitter();
        this.mouseDown = new Subject();
        this.mouseMove = new Subject();
        this.mouseUp = new Subject();
    }
    Draggable.prototype.ngOnInit = function () {
        var _this = this;
        var mouseDrag = this.mouseDown.flatMap(function (mouseDownEvent) {
            _this.dragStart.next({ x: 0, y: 0 });
            var currentDrag = new Subject();
            _this.draggableHelper.currentDrag.next(currentDrag);
            var mouseMove = _this.mouseMove
                .map(function (mouseMoveEvent) {
                mouseMoveEvent.preventDefault();
                return {
                    currentDrag: currentDrag,
                    x: mouseMoveEvent.clientX - mouseDownEvent.clientX,
                    y: mouseMoveEvent.clientY - mouseDownEvent.clientY
                };
            })
                .map(function (moveData) {
                if (_this.snapGrid.x) {
                    moveData.x = Math.floor(moveData.x / _this.snapGrid.x) * _this.snapGrid.x;
                }
                if (_this.snapGrid.y) {
                    moveData.y = Math.floor(moveData.y / _this.snapGrid.y) * _this.snapGrid.y;
                }
                return moveData;
            })
                .map(function (moveData) {
                if (!_this.dragAxis.x) {
                    moveData.x = 0;
                }
                if (!_this.dragAxis.y) {
                    moveData.y = 0;
                }
                return moveData;
            })
                .takeUntil(Observable.merge(_this.mouseUp, _this.mouseDown));
            mouseMove.takeLast(1).subscribe(function (_a) {
                var x = _a.x, y = _a.y;
                _this.dragEnd.next({ x: x, y: y });
                currentDrag.complete();
                _this.setCssTransform('');
            });
            return mouseMove;
        });
        mouseDrag.subscribe(function (_a) {
            var x = _a.x, y = _a.y, currentDrag = _a.currentDrag;
            _this.dragging.next({ x: x, y: y });
            _this.setCssTransform("translate(" + x + "px, " + y + "px)");
            currentDrag.next({ rectangle: _this.element.nativeElement.getBoundingClientRect(), dropData: _this.dropData });
        });
    };
    Draggable.prototype.ngOnDestroy = function () {
        this.mouseDown.complete();
        this.mouseMove.complete();
        this.mouseUp.complete();
    };
    /**
     * @private
     */
    Draggable.prototype.onMouseDown = function (event) {
        this.mouseDown.next(event);
    };
    /**
     * @private
     */
    Draggable.prototype.onMouseMove = function (event) {
        this.mouseMove.next(event);
    };
    /**
     * @private
     */
    Draggable.prototype.onMouseUp = function (event) {
        this.mouseUp.next(event);
    };
    Draggable.prototype.setCssTransform = function (value) {
        if (this.ghostDragEnabled) {
            this.renderer.setElementStyle(this.element.nativeElement, 'transform', value);
            this.renderer.setElementStyle(this.element.nativeElement, '-webkit-transform', value);
            this.renderer.setElementStyle(this.element.nativeElement, '-ms-transform', value);
            this.renderer.setElementStyle(this.element.nativeElement, '-moz-transform', value);
            this.renderer.setElementStyle(this.element.nativeElement, '-o-transform', value);
        }
    };
    Draggable.decorators = [
        { type: Directive, args: [{
                    selector: '[mwlDraggable]'
                },] },
    ];
    /** @nocollapse */
    Draggable.ctorParameters = [
        { type: ElementRef, },
        { type: Renderer, },
        { type: DraggableHelper, },
    ];
    Draggable.propDecorators = {
        'dropData': [{ type: Input },],
        'dragAxis': [{ type: Input },],
        'snapGrid': [{ type: Input },],
        'ghostDragEnabled': [{ type: Input },],
        'dragStart': [{ type: Output },],
        'dragging': [{ type: Output },],
        'dragEnd': [{ type: Output },],
        'onMouseDown': [{ type: HostListener, args: ['mousedown', ['$event'],] },],
        'onMouseMove': [{ type: HostListener, args: ['document:mousemove', ['$event'],] },],
        'onMouseUp': [{ type: HostListener, args: ['document:mouseup', ['$event'],] },],
    };
    return Draggable;
}());
//# sourceMappingURL=draggable.directive.js.map